/*
// www.lebois-racing.com
4 actuators SRT80
competition control box (3 buttons)
Leonardo only
v1.0
uploaded on 23/08/03
v1.1 : added support for SRT100 actuator as the 5th actuator
v1.2 : added extra safety for the calibration
*/

// WIRING : IT CAN'T BE CHANGED. SAME AS OPENSFX100
#define StepPin1 8
#define StepPin2 9
#define StepPin3 10
#define StepPin4 11
#define StepPin5 12

#define SignOutPin 13  //for tractionloss or securitymod

#define DirPin1 4
#define DirPin2 5
#define DirPin3 6
#define DirPin4 7
#define DirPin5 3

#define RelayPin      A0       //used to control the PSU thougth a relay
#define IncreaseGain  A1
#define DecreaseGain  A2
#define ConnectPin    A3
#define ConnectLed    A5
#define SecurityPin    2


//#define SRT100

#ifdef SRT100
#define midPoint 18350
#else
#define midPoint 32768
#endif

#define LEONARDO

#define pulseDelay 0  // mostly set the speed of the actuator : the lower the faster. Don't go below 2 or it will fail.
#define directionDelay 2
#define calibrationSpeed 10  //la vitesse de rotation : de 0 (rapide) à 1000 (lent)

int pinout[3] = { IncreaseGain, DecreaseGain, ConnectPin };
bool calibrated = false;
bool halfCalibrated = false;
bool invertCalibrationDirection = true;
#include "src/Joystick.h"
Joystick_ Joystick(0x06, JOYSTICK_TYPE_JOYSTICK, 3);
unsigned long lastPush[3] = { 0, 0, 0 };
unsigned long lastBtnState[3] = { 0, 0, 0 };



unsigned m1Target = 0, m2Target = 0, m3Target = 0, m4Target = 0, m5Target = 65535;
unsigned m1Position = 0, m2Position = 0, m3Position = 0, m4Position = 0, m5Position = 65535;
byte pulseM1 = B00000000, pulseM2 = B00000000, pulseM3 = B00000000, pulseM4 = B00000000, pulseM5 = B00000000;
int dir1 = -1, dir2 = -1, dir3 = -1, dir4 = -1, dir5 = -1;  //Will be used to set the motors direction
bool connected = false;
bool servoEnabled = false;

void setup() {

  Serial.begin(921600);  //To communicate with FlyPT. In FlyPT, put that number in "Serial speed".

  pinMode(IncreaseGain, INPUT_PULLUP);
  pinMode(DecreaseGain, INPUT_PULLUP);
  pinMode(ConnectPin, INPUT_PULLUP);
  pinMode(SecurityPin, INPUT_PULLUP);
  
  pinMode(ConnectLed, OUTPUT);
  digitalWrite(ConnectLed,LOW);

  Joystick.begin(false);

  pinMode(RelayPin, OUTPUT);
  digitalWrite(RelayPin, LOW);  //LOW disable the driver

  pinMode(StepPin1, OUTPUT);
  pinMode(DirPin1, OUTPUT);
  digitalWrite(DirPin1, HIGH);


  pinMode(StepPin2, OUTPUT);
  pinMode(DirPin2, OUTPUT);
  digitalWrite(DirPin2, HIGH);

  pinMode(StepPin3, OUTPUT);
  pinMode(DirPin3, OUTPUT);
  digitalWrite(DirPin3, HIGH);

  pinMode(StepPin4, OUTPUT);
  pinMode(DirPin4, OUTPUT);
  digitalWrite(DirPin4, HIGH);

  pinMode(StepPin5, OUTPUT);
  pinMode(DirPin5, OUTPUT);
  digitalWrite(DirPin5, HIGH);

  pinMode(SignOutPin, INPUT_PULLUP);

  disableServo();
}

void loop() {
  SerialReaderP();  //Get the datas from Simtools
  if(!calibrated){Calibrate();};
  moveMotor();
  sendConnectCommand();
}

void SerialReaderP() {
  byte commandbufferE[10] = { 0 };  // variables pour stocker les valeurs de l'accélérateur et des freins

  while (Serial.available()) {  //To communicate with FlyPT. In FlyPT, put that number in "Serial speed".
    char incomingChar = Serial.read();

    if (incomingChar == 'A') {enableServo();
      halfCalibrated =false;
      calibrated=false;
      m5Position=65535;
      m5Target = 65535;
      }

else if (incomingChar == 'T') {
  if(  !((PIND & (1 << PD1)) >> PD1)){Security();}  
      if (Serial.available() >= 9) {
        for (int i = 0; i < 10; i++) { commandbufferE[i] = Serial.read(); }
        if(calibrated){
        m1Target = commandbufferE[0] << 8 | commandbufferE[1];  //because the communication is 8bits, and the data are 16 bits, their is two bytes to combine
        m2Target = commandbufferE[2] << 8 | commandbufferE[3];
        m3Target = commandbufferE[4] << 8 | commandbufferE[5];
        m4Target = commandbufferE[6] << 8 | commandbufferE[7];
        m5Target = commandbufferE[8] << 8 | commandbufferE[9];}
        else{
        m1Target = commandbufferE[0] << 8 | commandbufferE[1];  //because the communication is 8bits, and the data are 16 bits, their is two bytes to combine
        m2Target = commandbufferE[2] << 8 | commandbufferE[3];
        m3Target = commandbufferE[4] << 8 | commandbufferE[5];
        m4Target = commandbufferE[6] << 8 | commandbufferE[7];
        //Calibrate();
        }}
         }
else if (incomingChar == 'P') {
  if(  !((PIND & (1 << PD1)) >> PD1)){Security();} 
      if (Serial.available() >= 7) {
        for (int i = 0; i < 8; i++) { commandbufferE[i] = Serial.read(); }
        m1Target = commandbufferE[0] << 8 | commandbufferE[1];  //because the communication is 8bits, and the data are 16 bits, their is two bytes to combine
        m2Target = commandbufferE[2] << 8 | commandbufferE[3];
        m3Target = commandbufferE[4] << 8 | commandbufferE[5];
        m4Target = commandbufferE[6] << 8 | commandbufferE[7];

      }}
     else if (incomingChar == 'C') {
      connected = false;
      disableServo();
      halfCalibrated =false;
      calibrated=false;
      m5Position=65535;
      m5Target = 65535;
    }
  }
}

void moveMotor() {
 #ifdef LEONARDO
  directionManagerLeonardo();  //set the directions of each motor
  singleStepLeonardo();        //make the motor move one step
  #else
directionManagerUno();
singleStepUno();
  #endif
}

void directionManagerLeonardo() {
  byte dirChange = 0;

  if ((m1Target > m1Position) && (dir1 == -1)) {  //pin4 /PD4 // on monte
    PORTD &= B11101111;                                       // dir = low
    dir1 = 1;
    dirChange = 1;
  }

  if ((m1Target < m1Position) && (dir1 == 1)) {
    PORTD |= B00010000;
    dir1 = -1;
    dirChange = 1;
  }

  if ((m2Target > m2Position) && (dir2 == -1)) {  //pin5
    PORTC &= B10111111;
    dir2 = 1;
    dirChange = 1;
  }

  if ((m2Target < m2Position) && (dir2 == 1)) {
    PORTC |= B01000000;
    dir2 = -1;
    dirChange = 1;
  }

  if ((m3Target > m3Position) && (dir3 == -1)) {  //pin6
    PORTD &= B01111111;
    dir3 = 1;
    dirChange = 1;
  }

  if ((m3Target < m3Position) && (dir3 == 1)) {
    PORTD |= B10000000;
    dir3 = -1;
    dirChange = 1;
  }

  if ((m4Target > m4Position) && (dir4 == -1)) {  //pin7
    PORTE &= B10111111;
    dir4 = 1;
    dirChange = 1;
  }

  if ((m4Target < m4Position) && (dir4 == 1)) {
    PORTE |= B01000000;
    dir4 = -1;
    dirChange = 1;
  }

  if ((m5Target > m5Position) && (dir5 == -1)) {
    digitalWrite(DirPin5, 0);
    dir5 = 1;
    dirChange = 1;
  }

  if ((m5Target < m5Position) && (dir5 == 1)) {
    digitalWrite(DirPin5, 1);
    dir5 = -1;
    dirChange = 1;
  }


  if (dirChange == 1) {
    delayMicroseconds(directionDelay);
  }
}

void singleStepLeonardo() {
  //pulse is 8, 9, 10, 11
  if (m1Target != m1Position) {  //pin8 /PB4
    pulseM1 = B00010000;
    m1Position += dir1;
  } else {
    pulseM1 = B00000000;
  }

  if (m2Target != m2Position) {  //pin9 /PB5
    pulseM2 = B00100000;
    m2Position += dir2;
  } else {
    pulseM2 = B00000000;
  }

  if (m3Target != m3Position) {  //pin10 / PB6
    pulseM3 = B01000000;
    m3Position += dir3;
  } else {
    pulseM3 = B00000000;
  }

  if (m4Target != m4Position) {  //pin11 / PB7
    pulseM4 = B10000000;
    m4Position += dir4;
  } else {
    pulseM4 = B00000000;
  }

  if (m5Target != m5Position) {
    pulseM5 = B01000000;
    m5Position += dir5;
  } else {
    pulseM5 = B00000000;
  }
  if ((pulseM1 == B00000000) && (pulseM2 == B00000000) && (pulseM3 == B00000000) && (pulseM4 == B00000000) && (pulseM5 == B00000000)) { return; }
  PORTB |= pulseM1 | pulseM2 | pulseM3 | pulseM4;
  PORTD |= pulseM5;
  delayMicroseconds(pulseDelay+!calibrated*calibrationSpeed);
  PORTB &= B00001111;
  PORTD &= B10111111;

}

void Calibrate(){
    
 //if (digitalRead(SignOutPin)==0){       //Si l'état change, on change la direction 
  if ( !(PINC & bit (7))) {
  digitalWrite(DirPin5,!invertCalibrationDirection );                  // On repart dans l'autre direction, c'est-à-dire qu'on monte, donc dir = low
  calibrated = true;
  m5Position=0;}
  else{
    m5Target = m5Position-1; // on teste avec -1 au lieu de +1
  }
 } 

void enableServo() {
  connected = true;
  digitalWrite(ConnectLed, connected);
  digitalWrite(RelayPin, LOW);
  servoEnabled = true;
}
void disableServo() {
  digitalWrite(RelayPin, HIGH);
  digitalWrite(ConnectLed, connected);
  servoEnabled = false;
}



void MoveOneStep() {
  digitalWrite(StepPin5, 1);
  digitalWrite(ConnectLed, 0);
  delay(20 + calibrationSpeed);
  digitalWrite(StepPin5, 0);
  digitalWrite(ConnectLed, 1);
  delay(20 + calibrationSpeed);
}

void sendConnectCommand() {  //PF1 4 5
  bool btnState[3] = { (PINF >> 1 & B01000000 >> 1), (PINF >> 5 & B00100000 >> 5), (PINF >> 4 & B00010000 >> 4) };
  for (byte i = 0; i < 3; i++) {
    if (!btnState[i]) {           //si bouton enfoncé
      if (lastBtnState[i] == 0) {
        lastBtnState[i] = 1;
        Joystick.setButton(i, 1);
        Joystick.sendState();
        lastPush[i] = millis();}
      } 
    if(btnState[i] && lastBtnState[i] == 1 ){
      if(millis() - lastPush[i] > 100) {
            lastBtnState[i] = 0;
            Joystick.setButton(i, 0);
            Joystick.sendState();
          }
  }
    }
  }

  void Security(){
    disableServo();
    while(1==1){
      digitalWrite(ConnectLed,0);
      delay(200);
      digitalWrite(ConnectLed,1);
      delay(200);
    }
  }

void directionManagerUno() { 
    byte dirChange = 0;

  if ((m1Target > m1Position) && (dir1 == -1)) {
    PORTD &= B11101111;
    dir1 = 1;
    dirChange = 1;
  }

  if ((m1Target < m1Position ) && (dir1 == 1)) {
    PORTD |= B00010000;
    dir1 = -1;
    dirChange = 1;
  }

  if ((m2Target > m2Position) && (dir2 == -1)) {
    PORTD &= B11011111;
    dir2 = 1;
    dirChange = 1;
  }
  
  if ((m2Target < m2Position ) && (dir2 == 1)) {
    PORTD |= B00100000;
    dir2 = -1;
    dirChange = 1;
  }

  if ((m3Target > m3Position) && (dir3 == -1)) {
    PORTD &= B10111111;
    dir3 = 1;
    dirChange = 1;
  }
  
  if ((m3Target < m3Position ) && (dir3 == 1)) {
    PORTD |= B01000000;
    dir3 = -1;
    dirChange = 1;
  }

  if ((m4Target > m4Position) && (dir4 == -1)) {
    PORTD &= B01111111;
    dir4 = 1;
    dirChange = 1;
  }
  
  if ((m4Target < m4Position ) && (dir4 == 1)) {
    PORTD |= B10000000;
    dir4 = -1;
    dirChange = 1;
  }
  
  if (dirChange == 1) {
    delayMicroseconds(directionDelay);
    dirChange = 0;
  }
}


void singleStepUno() {
  //pulse is 8, 9, 10, 11
  if (m1Target != m1Position) {
    pulseM1 = B00000001;
    m1Position += dir1;
  }
  else{pulseM1 = B00000000;}
  
  if (m2Target != m2Position) {
    pulseM2 = B00000010;
    m2Position += dir2;
  }
  else{pulseM2 = B00010000;}
    
  if (m3Target != m3Position) {
        pulseM3 = B00000100;
    m3Position += dir3;
  }
  else{pulseM3 = B00000000;}

  if (m4Target != m4Position) {
        pulseM4 = B00001000;
    m4Position += dir4;
  }
  else{pulseM4 = B00000000;}
    
  if((pulseM1 == B00000000) &&(pulseM2 == B00000000)&&(pulseM3 == B00000000)&& (pulseM4 == B00000000)){return;}
      PORTB |= pulseM1 | pulseM2 | pulseM3 | pulseM4;
      delayMicroseconds(pulseDelay);
      PORTB &= B11110000;
}
